
%% You can run this file in cell mode: each cell (starting with %%) can be run by pressing CTRL+ENTER inside it

%% Initialization

% sampling interval
Ts = 0.05;
% create model
% the reward function is "normalized LQR" = -x' Qrew x - u' Rrew u, scaled and translated into [0,1]
% maxu is the maximum input voltage, Ts is the sampling period
model = ipsetup_problem('model', 'rewtype=nlqr Qrew=[1,0;0,0.01] Rrew=0.3', 'type=ip maxu=3', sprintf('Ts=%f', Ts));

% set the discount factor
discount = 0.9;

%% Run the uniform planner

% configure the planner

% for the return study:
N = [50 100:100:500];
% for the exec time study:
% N = [100 500 1000 2000];
D = 0 * N;
RET = 0 * N;
E = 0*N;
for i = 1:length(N),

    cfg = struct;
    cfg.n = N(i);           % budget of node expansions at each step
    cfg.U = [-3 0 3];       % discretized action space, in volts for the inv pend
    cfg.discount = discount;

    % example of running the uniform planner in closed loop
    x0 = [-pi; 0];          % initial state = pointing down
    K = 60;
    X = NaN(2, K+1); U = NaN(1, K+1); R = NaN(1, K+1);
    X(:, 1) = x0;
    % visualize the initial state (comment these lines out if you don't want the picture)
%     viscfg = struct;
%     viscfg.model = model; viscfg.trial = 1; viscfg.mpcstep = 1;
%     viscfg.X = X; viscfg.k = 0;
%     [figh, viscfg.gview] = dc_ip_visualize(viscfg);    % note view is stored, else it's recreated each time
    for k = 1:K,
        % run the planner
        [U(:, k), planinfo] = upd(X(:, k), model, cfg);
        D(i) = D(i) + planinfo.depth;
        E(i) = E(i) + planinfo.trun;
        % see upd code for the statistics you can examine in planinfo
        [X(:, k+1), R(k+1)] = feval(model.fun, model, X(:, k), U(:, k));
%         % visualize updated state (comment these lines out if you don't want the picture)
%         viscfg.X = X; viscfg.k = k;
%         [figh, viscfg.gview] = dc_ip_visualize(viscfg);
    end;
    D(i) = D(i)/K;
    E(i) = E(i)/K;
    RET(i) = sum(discount.^(0:K-1) .* R(2:K+1));
    
end;

%% Run the optimistic planner
% for the return study:
N = [50 100:100:500];
% for the exec time study:
% N = [100 500 1000 2000];
DO = 0 * N;
EO = 0 * N;
RETO = 0 * N;
for i = 1:length(N),

    % configure the planner
    cfg = struct;
    cfg.n = N(i);            % budget of node expansions at each step
    cfg.U = [-3 0 3];       % discretized action space, in volts for the inv pend
    cfg.discount = discount;

    % example of running the uniform planner in closed loop
    x0 = [-pi; 0];          % initial state = pointing down
    K = 60;
    X = NaN(2, K+1); U = NaN(1, K+1); R = NaN(1, K+1);
    X(:, 1) = x0;
%     % visualize the initial state (comment these lines out if you don't want the picture)
%     viscfg = struct;
%     viscfg.model = model; viscfg.trial = 1; viscfg.mpcstep = 1;
%     viscfg.X = X; viscfg.k = 0;
%     [figh, viscfg.gview] = dc_ip_visualize(viscfg);    % note view is stored, else it's recreated each time
    for k = 1:K,
        % run the planner
        [U(:, k), planinfo] = opd(X(:, k), model, cfg);
        DO(i) = DO(i) + planinfo.depth;
        EO(i) = EO(i) + planinfo.trun;
        % see upd code for the statistics you can examine in planinfo
        [X(:, k+1), R(k+1)] = feval(model.fun, model, X(:, k), U(:, k));
%         % visualize updated state (comment these lines out if you don't want the picture)
%         viscfg.X = X; viscfg.k = k;
%         [figh, viscfg.gview] = dc_ip_visualize(viscfg);
    end;
    DO(i) = DO(i)/K;
    EO(i) = EO(i)/K;
    RETO(i) = sum(discount.^(0:K-1) .* R(2:K+1));

end;
%% Plot the comparison

figure; 
subplot(311);
plot(N, RET, 'k-o', N, RETO, 'b-*'); ylabel('Return'); legend('Uniform', 'Optimistic');
subplot(312);
plot(N, D, 'k-o', N, DO, 'b-*'); ylabel('Depth reached');
subplot(313);
plot(N, E, 'k-o', N, EO, 'b-*'); ylabel('Exec time');
xlabel('Budget n');

